@@ -1,11 +1,11 @@
-from .mwedittypes import EditTypes
+from .mwedittypes import StructuredEditTypes, SimpleEditTypes
 
 __title__ = "mwedittypes"
 __summary__ = "mwedittypes is a package that supports edit diffs and action detection for Wikipedia"
 __url__ = "https://github.com/geohci/edit-types"
 
-__version__ = "1.1.5"
+__version__ = "2.0.0"
 
 __license__ = "MIT License"
 
-__all__ = ["EditTypes"]
+__all__ = ["StructuredEditTypes", "SimpleEditTypes"]
